(function() {
  var $, Builder, EventEmitter, assetTypes, emitters, fs, jsdom, parserPath, patcher, path, utils, _;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  EventEmitter = require("events").EventEmitter;
  _ = require("underscore");
  emitters = {};
  fs = require("fs");
  path = require("path");
  jsdom = require("jsdom");
  patcher = require("./patcher");
  parserPath = "" + __dirname + "/parsers";
  utils = require("./utils");
  jsdom = patcher.patch(jsdom);
  $ = require("jquery");
  assetTypes = {
    js: "script",
    css: "link",
    images: "img"
  };
  Builder = (function() {
    function Builder(file, public, options) {
      this.file = file;
      this.public = public;
      this.options = options != null ? options : {};
      this.directory = path.dirname(file);
      this.emitter = new EventEmitter();
      this.root = options.root ? file : false;
    }
    Builder.prototype.build = function(callback) {
      var emitter;
      emitter = this.emitter;
      emitter.once("read", __bind(function(code) {
        this.document = jsdom(code);
        return this.flatten(this.document, this.directory, emitter);
      }, this));
      emitter.once("flattened", __bind(function() {
        return this.bundle(emitter);
      }, this));
      emitter.once("bundled", function() {
        return emitter.emit("built");
      });
      emitter.once("built", __bind(function() {
        return callback(null, this.document.innerHTML);
      }, this));
      return this.read(this.file);
    };
    Builder.prototype.read = function(file) {
      return fs.readFile(file, "utf8", __bind(function(err, code) {
        if (err) {
          throw err;
        }
        return this.emitter.emit("read", code);
      }, this));
    };
    Builder.prototype.flatten = function(document, directory, emitter) {
      var CommentParser, builder, parser;
      builder = this;
      CommentParser = require(parserPath + "/comments");
      parser = new CommentParser(this.document, this.directory);
      return parser.parse(this.document, this.directory, function(document) {
        return emitter.emit("flattened");
      });
    };
    Builder.prototype.bundle = function(emitter) {
      var callback, document, elements, finished, parser, tag, type, _results;
      document = this.document;
      finished = utils.countdown(_.size(assetTypes));
      callback = __bind(function(err) {
        if (err) {
          throw err;
        }
        if (finished()) {
          return emitter.emit("bundled");
        }
      }, this);
      _results = [];
      for (type in assetTypes) {
        tag = assetTypes[type];
        elements = $(tag, document).get();
        if (elements.length === 0) {
          if (finished()) {
            emitter.emit("bundled");
            return;
          } else {
            continue;
          }
        }
        parser = require(parserPath + ("/" + type));
        _results.push(parser.build(elements, this.public, this.directory, callback));
      }
      return _results;
    };
    return Builder;
  })();
  module.exports = Builder;
}).call(this);
