(function() {
  var Bundler, EventEmitter, fs, lib, path, plugin, utils, _;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  _ = require("underscore");
  path = require("path");
  lib = "" + __dirname + "/..";
  utils = require("" + lib + "/utils");
  plugin = require("" + lib + "/plugin");
  EventEmitter = require("events").EventEmitter;
  fs = require("fs");
  Bundler = (function() {
    function Bundler(buildName, public, main) {
      this.buildName = buildName;
      this.public = public;
      this.main = main;
    }
    Bundler.prototype.bundle = function(assets, output) {
      var buildName, elem, emitter, ext, sources;
      buildName = this.buildName;
      emitter = new EventEmitter();
      ext = path.extname(buildName).substring(1);
      this.attribute = ext === "css" ? "href" : "src";
      sources = (function() {
        var _i, _len, _results;
        _results = [];
        for (_i = 0, _len = assets.length; _i < _len; _i++) {
          elem = assets[_i];
          _results.push(this.main + "/" + elem[this.attribute]);
        }
        return _results;
      }).call(this);
      emitter.once("read", __bind(function(files) {
        return this.render(files, emitter);
      }, this));
      emitter.once("rendered", __bind(function(buildFile) {
        buildFile = _.values(buildFile).join("\n");
        return this.write(buildFile, this.public, emitter);
      }, this));
      emitter.once("written", __bind(function(buildPath) {
        return this.modify(assets, emitter);
      }, this));
      emitter.once("modified", function() {
        return output(null);
      });
      return this.read(sources, emitter);
    };
    Bundler.prototype.read = function(assets, emitter) {
      return utils.readFiles(assets, function(err, files) {
        if (err) {
          throw err;
        }
        return emitter.emit("read", files);
      });
    };
    Bundler.prototype.render = function(files, emitter) {
      var buildFile, code, file, finished, p, _results;
      buildFile = utils.toKeys(_.keys(files));
      finished = utils.countdown(_.size(buildFile));
      _results = [];
      for (file in files) {
        code = files[file];
        p = plugin(file);
        _results.push(p ? p.render(code, file, function(err, code) {
          buildFile[file] = code;
          if (finished()) {
            return emitter.emit("rendered", buildFile);
          }
        }) : (buildFile[file] = code, finished() ? emitter.emit("rendered", buildFile) : void 0));
      }
      return _results;
    };
    Bundler.prototype.write = function(content, to, emitter) {
      var fullPath;
      fullPath = to + "/" + this.buildName;
      return fs.writeFile(fullPath, content, "utf8", function(err) {
        if (err) {
          throw err;
        }
        return emitter.emit("written", fullPath);
      });
    };
    Bundler.prototype.modify = function(assets, emitter) {
      var asset, assetsLength, i, _len;
      assetsLength = assets.length;
      for (i = 0, _len = assets.length; i < _len; i++) {
        asset = assets[i];
        if (i === assetsLength - 1) {
          asset[this.attribute] = this.buildName;
        } else {
          if (asset.parentNode) {
            asset.parentNode.removeChild(asset);
          }
        }
      }
      return emitter.emit("modified");
    };
    return Bundler;
  })();
  module.exports = function(buildName, publicDir, mainDir) {
    return new Bundler(buildName, publicDir, mainDir);
  };
}).call(this);
